@@ -2,7 +2,6 @@
 #include "helper.h"
 #include "any_missing.h"
 #include "is_integerish.h"
-#include "cmessages.h"
 
 typedef enum {
     CL_LOGICAL, CL_INTEGER, CL_INTEGERISH, CL_NUMERIC, CL_DOUBLE, CL_STRING, CL_LIST, CL_COMPLEX,
@@ -31,6 +30,12 @@ typedef struct {
     bound_t upper;
 } checker_t;
 
+typedef struct {
+    Rboolean ok;
+    char msg[255];
+} msg_t;
+
+
 static inline Rboolean ii_eq(const R_xlen_t x, const R_xlen_t y) { return x == y; }
 static inline Rboolean ii_lt(const R_xlen_t x, const R_xlen_t y) { return x <  y; }
 static inline Rboolean ii_gt(const R_xlen_t x, const R_xlen_t y) { return x >  y; }
@@ -63,6 +68,18 @@ static const char * CLSTR[] = {
      "atomic", "atomic vector", "matrix", "data frame", "environment", "function", "NULL"
 };
 
+static const msg_t MSGT = { .ok = TRUE };
+static const msg_t MSGF = { .ok = FALSE };
+
+static msg_t message(const char *fmt, ...) {
+    msg_t msg = { .ok = FALSE };
+    va_list vargs;
+    va_start(vargs, fmt);
+    vsnprintf(msg.msg, 255, fmt, vargs);
+    va_end(vargs);
+    return msg;
+}
+
 /*********************************************************************************************************************/
 /* Some helper functions                                                                                             */
 /*********************************************************************************************************************/
@@ -71,15 +88,20 @@ static msg_t check_bound(SEXP x, const bound_t bound) {
         const double *xp = REAL(x);
         const double * const xend = xp + xlength(x);
         for (; xp != xend; xp++) {
-            if (!ISNAN(*xp) && !bound.fun(*xp, bound.cmp))
-                return make_msg("All elements must be %s %g", CMPSTR[bound.op], bound.cmp);
+            if (!ISNAN(*xp) && !bound.fun(*xp, bound.cmp)) {
+                if (bound.cmp == R_PosInf)
+                    return message("All elements must be %s Inf", CMPSTR[bound.op]);
+                if (bound.cmp == R_NegInf)
+                    return message("All elements must be %s -Inf", CMPSTR[bound.op]);
+                return message("All elements must be %s %g", CMPSTR[bound.op], bound.cmp);
+            }
         }
     } else if (isInteger(x)) {
         const int *xp = INTEGER(x);
         const int * const xend = xp + xlength(x);
         for (; xp != xend; xp++) {
             if (*xp != NA_INTEGER && !bound.fun((double) *xp, bound.cmp))
-                return make_msg("All elements must be %s %g", CMPSTR[bound.op], bound.cmp);
+                return message("All elements must be %s %g", CMPSTR[bound.op], bound.cmp);
         }
     } else if (isString(x)) {
         const R_xlen_t nx = xlength(x);
@@ -87,7 +109,7 @@ static msg_t check_bound(SEXP x, const bound_t bound) {
         for (R_xlen_t i = 0; i < nx; i++) {
             nchar = STRING_ELT(x, i) == NA_STRING ? 0. : (double)length(STRING_ELT(x, i));
             if (!bound.fun(nchar, bound.cmp))
-                return make_msg("All elements must have %s %g chars", CMPSTR[bound.op], bound.cmp);
+                return message("All elements must have %s %g chars", CMPSTR[bound.op], bound.cmp);
         }
     } else {
         error("Bound checks only possible for numeric variables and strings");
@@ -351,15 +373,15 @@ static void parse_rule(checker_t *checker, const char *rule) {
 /*********************************************************************************************************************/
 static msg_t check_rule(SEXP x, const checker_t *checker, const Rboolean err_msg) {
     if (checker->class.fun != NULL && !checker->class.fun(x)) {
-        return err_msg ? make_msg("Must be of class '%s', not '%s'", CLSTR[checker->class.name], guessType(x)) : MSGF;
+        return err_msg ? message("Must be of class '%s', not '%s'", CLSTR[checker->class.name], guessType(x)) : MSGF;
     }
 
     if (checker->missing.fun != NULL && checker->missing.fun(x)) {
-        return err_msg ? make_msg("May not contain missing values") : MSGF;
+        return err_msg ? message("May not contain missing values") : MSGF;
     }
 
     if (checker->len.fun != NULL && !checker->len.fun(xlength(x), checker->len.cmp)) {
-        return err_msg ? make_msg("Must be of length %s %i, but has length %g", CMPSTR[checker->len.op], checker->len.cmp, (double)xlength(x)) : MSGF;
+        return err_msg ? message("Must be of length %s %i, but has length %g", CMPSTR[checker->len.op], checker->len.cmp, (double)xlength(x)) : MSGF;
     }
 
     if (checker->lower.fun != NULL) {
@@ -435,6 +457,7 @@ SEXP c_qassert(SEXP x, SEXP rules, SEXP recursive) {
     } else {
         failed = qassert1(x, checker, result, nrules);
     }
+
     if (failed == 0)
         return ScalarLogical(TRUE);
 
@@ -460,14 +483,26 @@ static inline Rboolean qtest1(SEXP x, const checker_t *checker, const R_len_t nr
     return FALSE;
 }
 
-static inline Rboolean qtest_list(SEXP x, const checker_t *checker, const R_len_t nrules) {
+static inline Rboolean qtest_list(SEXP x, const checker_t *checker, const R_len_t nrules, R_len_t depth) {
     if (!isNewList(x) || isNull(x))
         error("Argument 'x' must be a list or data.frame");
 
     const R_len_t nx = xlength(x);
-    for (R_xlen_t i = 0; i < nx; i++) {
-        if (!qtest1(VECTOR_ELT(x, i), checker, nrules))
-            return FALSE;
+    if (depth > 1) {
+        for (R_xlen_t i = 0; i < nx; i++) {
+            if (isRList(VECTOR_ELT(x, i))) {
+                if (!qtest_list(VECTOR_ELT(x, i), checker, nrules, depth - 1))
+                    return FALSE;
+            } else {
+                if (!qtest1(VECTOR_ELT(x, i), checker, nrules))
+                    return FALSE;
+            }
+        }
+    } else {
+        for (R_xlen_t i = 0; i < nx; i++) {
+            if (!qtest1(VECTOR_ELT(x, i), checker, nrules))
+                return FALSE;
+        }
     }
     return TRUE;
 }
@@ -479,7 +514,7 @@ Rboolean qtest(SEXP x, const char *rule) {
     return qtest1(x, &checker, 1);
 }
 
-SEXP c_qtest(SEXP x, SEXP rules, SEXP recursive) {
+SEXP c_qtest(SEXP x, SEXP rules, SEXP recursive, SEXP depth) {
     const R_len_t nrules = length(rules);
 
     if (!isString(rules))
@@ -496,7 +531,7 @@ SEXP c_qtest(SEXP x, SEXP rules, SEXP recursive) {
         parse_rule(&checker[i], CHAR(STRING_ELT(rules, i)));
     }
 
-    return LOGICAL(recursive)[0] ?
-        ScalarLogical(qtest_list(x, checker, nrules)) :
-        ScalarLogical(qtest1(x, checker, nrules));
+    if (LOGICAL(recursive)[0])
+        return ScalarLogical(qtest_list(x, checker, nrules, asCount(depth, "depth")));
+    return ScalarLogical(qtest1(x, checker, nrules));
 }
